@@ -6,6 +6,7 @@
 #include "log.h"
 #include "connections.h"
 #include "plugin.h"
+#include "plugin_config.h"
 #include "sock_addr.h"
 
 #include "network_write.h"
@@ -29,6 +30,8 @@
 #define unsetenv(name)                _putenv_s((name), "")
 #endif
 
+static int network_mptcp = 0;
+
 void
 network_accept_tcp_nagle_disable (const int fd)
 {
@@ -539,6 +542,19 @@ static int network_server_init(server *srv, const network_socket_config *s, buff
 	} else
 #endif
 	{
+	  #ifdef __linux__
+		if (network_mptcp) {
+			/* manually define mptcp protocol number in case the compiler is using an older version of libc */
+			#ifndef IPPROTO_MPTCP
+			#define IPPROTO_MPTCP 262
+			#endif
+			if (-1 == (srv_socket->fd = fdevent_socket_nb_cloexec(family, SOCK_STREAM, IPPROTO_MPTCP))) {
+				log_pdebug(srv->errh, __FILE__, __LINE__, "socket() IPPROTO_MPTCP");
+				network_mptcp = 0;
+			}
+		}
+		if (-1 != srv_socket->fd) { } else /*fallback to tcp*/
+	  #endif
 		if (-1 == (srv_socket->fd = fdevent_socket_nb_cloexec(family, SOCK_STREAM, IPPROTO_TCP))) {
 		  #ifndef _WIN32
 			/* some configs might always include IPv6 addresses,
@@ -819,6 +835,8 @@ int network_init(server *srv, int stdin_fd) {
             network_merge_config(&p->defaults, cpv);
     }
 
+    network_mptcp = config_feature_bool(srv, "server.network-mptcp", 0);
+
     if (config_feature_bool(srv, "server.graceful-restart-bg", 0))
         srv->srvconf.systemd_socket_activation = 1;
 
