@@ -5,40 +5,9 @@ use IO::Socket ();
 use Test::More; # diag()
 use Socket;
 use Cwd 'abs_path';
+use Config;
 
-sub find_program {
-	my @DEFAULT_PATHS = ('/usr/bin/', '/usr/local/bin/');
-	my ($envname, $program) = @_;
-	my $location;
-
-	if (defined $ENV{$envname}) {
-		$location = $ENV{$envname};
-	} else {
-		$location = `which "$program" 2>/dev/null`;
-		chomp $location;
-		if (! -x $location) {
-			for my $path (@DEFAULT_PATHS) {
-				$location = $path . $program;
-				last if -x $location;
-			}
-		}
-	}
-
-	if (-x $location) {
-		$ENV{$envname} = $location;
-		return 1;
-	} else {
-		delete $ENV{$envname};
-		return 0;
-	}
-}
-
-BEGIN {
-	our $HAVE_PERL = find_program('PERL', 'perl');
-	if (!$HAVE_PERL) {
-		die "Couldn't find path to perl, but it obviously seems to be running";
-	}
-}
+$ENV{PERL} = $Config{perlpath} . $Config{_exe};
 
 sub mtime {
 	my $file = shift;
@@ -76,6 +45,18 @@ sub new {
 		$self->{LIGHTTPD_PATH} = $ENV{LIGHTTPD_EXE_PATH};
 	}
 
+	# test for cygwin w/ _WIN32 native lighttpd.exe (not linked w/ cygwin1.dll)
+	#   ($^O eq "MSWin32") is untested; not supported
+	$self->{"win32native"} = $^O eq "cygwin"
+	                      && 0 != system("ldd '$$self{LIGHTTPD_PATH}' | grep -q cygwin");
+	if ($^O eq "msys" && 0 != system("ldd '$$self{LIGHTTPD_PATH}' | grep -q msys-")) {
+		$self->{"win32native"} = 1;
+		# Note: msys2 mingw cross compile/link hangs if MSYS_NO_PATHCONV is set,
+		#       so scope setting MSYS_NO_PATHCONV here for running tests
+		$ENV{MSYS_NO_PATHCONV} = 1;
+		#$ENV{MSYS2_ARG_CONV_EXCL} = "*";
+	}
+
 	my ($name, $aliases, $addrtype, $net) = gethostbyaddr(inet_aton("127.0.0.1"), AF_INET);
 
 	$self->{HOSTNAME} = $name;
@@ -138,11 +119,13 @@ sub stop_proc {
 				chomp($winpid);
 				close($WH);
 			}
+			my $msys = ($^O eq "msys");
+			my $taskkill = $msys ? "/c/Windows/System32/taskkill.exe" : "/cygdrive/c/windows/system32/taskkill.exe";
 			if ($winpid) {
-				system('/cygdrive/c/windows/system32/taskkill.exe', '/F', '/T', '/PID', $winpid);
+				system($taskkill, '/F', '/T', '/PID', $winpid);
 			}
 			else {
-				system('/cygdrive/c/windows/system32/taskkill.exe', '/F', '/T', '/IM', 'lighttpd.exe');
+				system($taskkill, '/F', '/T', '/IM', 'lighttpd.exe');
 			}
 		}
 		else {
@@ -220,42 +203,6 @@ sub start_proc {
 	my $SOCK;
 	($SOCK, $self->{PORT}) = bind_ephemeral_tcp_socket();
 
-	my $testdir      = $self->{TESTDIR};
-	my $conf         = $self->{TESTDIR}.'/'.$self->{CONFIGFILE};
-	my $modules_path = $self->{MODULES_PATH};
-
-	# test for cygwin w/ _WIN32 native lighttpd.exe (not linked w/ cygwin1.dll)
-	#   ($^O eq "MSWin32") is untested; not supported
-	my $win32native = $^O eq "cygwin"
-	               && 0 != system("ldd '$$self{LIGHTTPD_PATH}' | grep -q cygwin");
-	$self->{"win32native"} = $win32native;
-
-	if ($win32native) {
-		$ENV{SHELL}         = "/bin/sh";
-		$ENV{PERL}          = cygpath_alm($ENV{PERL});
-		$testdir            = cygpath_alm($testdir);
-		$conf               = cygpath_alm($conf);
-		$modules_path       = cygpath_alm($modules_path);
-
-		$ENV{CYGROOT}       = cygpath_alm("/", 1);
-		$ENV{CYGVOL}        = $ENV{CYGROOT} =~ m%^([a-z]):%i
-		                      ? "/cygdrive/$1"
-		                      : "/cygdrive/c";
-	}
-
-	my @cmdline = ($self->{LIGHTTPD_PATH}, "-D", "-f", $conf, "-m", $modules_path);
-	splice(@cmdline, -2) if exists $ENV{LIGHTTPD_EXE_PATH};
-	if (!defined $ENV{"TRACEME"}) {
-	} elsif ($ENV{"TRACEME"} eq 'strace') {
-		@cmdline = (qw(strace -tt -s 4096 -o strace -f -v), @cmdline);
-	} elsif ($ENV{"TRACEME"} eq 'truss') {
-		@cmdline = (qw(truss -a -l -w all -v all -o strace), @cmdline);
-	} elsif ($ENV{"TRACEME"} eq 'gdb') {
-		@cmdline = ('gdb', '--batch', '--ex', 'run', '--ex', 'bt full', '--args', @cmdline);
-	} elsif ($ENV{"TRACEME"} eq 'valgrind') {
-		@cmdline = (qw(valgrind --tool=memcheck --track-origins=yes --show-reachable=yes --leak-check=yes --log-file=valgrind.%p), @cmdline);
-	}
-	# diag("\nstarting lighttpd at :$$self{PORT}, cmdline: @cmdline");
 	my $child = fork();
 	if (not defined $child) {
 		diag("\nFork failed");
@@ -263,8 +210,24 @@ sub start_proc {
 		return -1;
 	}
 	if ($child == 0) {
-		$ENV{'SRCDIR'} = $testdir;
-		if ($win32native) {
+
+		my $testdir      = $self->{TESTDIR};
+		my $conf         = $self->{TESTDIR}.'/'.$self->{CONFIGFILE};
+		my $modules_path = $self->{MODULES_PATH};
+
+		if ($self->{"win32native"}) {
+			$ENV{SHELL}         = "/bin/sh";
+			$ENV{PERL}          = cygpath_alm($ENV{PERL});
+			$testdir            = cygpath_alm($testdir);
+			$conf               = cygpath_alm($conf);
+			$modules_path       = cygpath_alm($modules_path);
+
+			my $msys = ($^O eq "msys");
+			$ENV{CYGROOT}       = cygpath_alm("/", 1);
+			$ENV{CYGVOL}        = $ENV{CYGROOT} =~ m%^([a-z]):%i
+			                      ? $msys ? "/$1" : "/cygdrive/$1"
+			                      : $msys ? "/c"  : "/cygdrive/c";
+
 			# On platforms where systemd socket activation is not supported
 			# or inconvenient for testing (i.e. cygwin <-> native Windows exe),
 			# there is a race condition with close() before server start,
@@ -299,6 +262,22 @@ BIND_OVERRIDE
 				fcntl($SOCK, Fcntl::F_SETFD(), 0); # clr FD_CLOEXEC
 			}
 		}
+
+		$ENV{'SRCDIR'} = $testdir;
+
+		my @cmdline = ($self->{LIGHTTPD_PATH}, "-D", "-f", $conf, "-m", $modules_path);
+		splice(@cmdline, -2) if exists $ENV{LIGHTTPD_EXE_PATH};
+		if (!defined $ENV{"TRACEME"}) {
+		} elsif ($ENV{"TRACEME"} eq 'strace') {
+			@cmdline = (qw(strace -tt -s 4096 -o strace -f -v), @cmdline);
+		} elsif ($ENV{"TRACEME"} eq 'truss') {
+			@cmdline = (qw(truss -a -l -w all -v all -o strace), @cmdline);
+		} elsif ($ENV{"TRACEME"} eq 'gdb') {
+			@cmdline = ('gdb', '--batch', '--ex', 'run', '--ex', 'bt full', '--args', @cmdline);
+		} elsif ($ENV{"TRACEME"} eq 'valgrind') {
+			@cmdline = (qw(valgrind --tool=memcheck --track-origins=yes --show-reachable=yes --leak-check=yes --log-file=valgrind.%p), @cmdline);
+		}
+		#diag("\nstarting lighttpd at :$$self{PORT}, cmdline: @cmdline");
 		#diag(sprintf('\ncmd: %s', "@cmdline"));
 		exec @cmdline or die($?);
 	}
@@ -361,7 +340,9 @@ sub handle_http {
 			print $remote "\012";
 		}
 	}
-	if ($^O ne "openbsd" && $^O ne "dragonfly" && !$self->{"win32native"}) {
+	if ($^O ne "openbsd" && $^O ne "dragonfly"
+	    && $Config{"archname"} !~ /^s390x-/
+	    && !$self->{"win32native"}) {
 		# (avoid on OS where TCP half-close may be reported as POLLHUP)
 		shutdown($remote, 1); # I've stopped writing data
 	}
